import asyncio, sys, os, signal, traceback
from datetime import timezone

from smsgateway import sink_sms
from smsgateway.sources.utils import *
from smsgateway.config import *

from telethon import TelegramClient, events
from telethon.tl.types import Chat, User, Channel, \
    PeerUser, PeerChat, PeerChannel
from telethon.tl.functions.users import GetFullUserRequest

from smsgateway.sources.telegram_utils import *


def init():
    global IDENTIFIER, app_log, api_id, api_hash, session_path, client

    app_log = setup_logging("telegram")
    IDENTIFIER = "TG"

    api_id = 242101
    api_hash = "80cbc97ce425aae38c1e0291ef2ab2a4"
    session_path = os.path.join(CONFIG_DIR, 'telegram')

    client = TelegramClient(session_path, api_id, api_hash)  # ,update_workers=1, spawn_read_thread=False)


if __name__ == '__main__':
    init()
    if not client.start():
        app_log.error("Could not connect! Run python3 -m smsgateway.sources.telegram to authorize!")
        sys.exit(1)

    app_log.info("Started TelegramClient")


@client.on(events.NewMessage())
@client.on(events.MessageEdited())
async def callback(event):
    chat_info = {'ID': event.message.id}
    try:
        if event.message.out:
            if not event.message.to_id:
                raise Exception("No to_id given, but messsage is going out!")
            chat_info.update(await get_outgoing_info(client, app_log, event.message.to_id))
        else:  # in
            chat_info.update(await get_incoming_info(client, app_log, event.message.from_id, event.message.to_id))

        if event.message.edit_date:
            chat_info['edit'] = "True"
        if event.message.date:
            date = event.message.date
            app_log.debug(date)
            date_utc = date.replace(tzinfo=timezone.utc)
            chat_info['date'] = date_utc.timestamp() #..strftime('%s')
        msg = ""

        if event.message.fwd_from:
            fwd_from = event.message.fwd_from
            user_info = {}
            if fwd_from.from_id:
                entity = await client.get_entity(fwd_from.from_id)
                (user_id, name, phone) = get_user_info(entity)
                user_info['user_id'] = user_id
                if name:
                    user_info['name'] = name
            elif fwd_from.channel_id:
                entity = await client.get_entity(client, fwd_from.channel_id)
                user_info['name'] = entity.title
            if 'name' in user_info:
                msg += f"Forwarded from {user_info['name']}:\n"
                chat_info.update({
                    'fwd': json.dumps(user_info)
                })

        if event.message.reply_to_msg_id:
            try:
                msg += await parseReplyTo(client, app_log, event)
            except Exception as e:
                msg += f"Reply parsing failed: {e}\n\n"
                app_log.warning(e, exc_info=True)
                app_log.warning(event.stringify())

        if event.message.reply_markup:
            try:
                chat_info.update(parseButtons(event.message.reply_markup))
            except Exception as e:
                msg += f"Button parsing failed: {e}\n\n"
                app_log.warning(e, exc_info=True)
                app_log.warning(event.stringify())

        msg += event.message.message

        media = event.message.media
        if media:
            if msg:
                msg += "\n"
            try:
                msg += parseMedia(media)
            except Exception as e:
                msg += f"Media parsing failed: {e}"

        if event.message.out:
            if 'to' in chat_info:
                app_log.info("New message to %s!" % chat_info['to'])
            sink_sms.send_dict(IDENTIFIER, msg, chat_info)
            # sink_sms.send_from_me(IDENTIFIER, msg, chat_info['to'])
        else:
            if 'from' in chat_info:
                app_log.info("New message from %s!" % chat_info['from'])
            sink_sms.send_dict(IDENTIFIER, msg, chat_info)
            # sink_sms.send(IDENTIFIER, msg, user_name, user_number, group_name)
        app_log.debug(msg)
    except Exception as e:
        event = event.stringify()
        trace = traceback.format_exc()
        app_log.warning(event, exc_info=True)
        app_log.warning(str(chat_info))
        sink_sms.send_notif("Telegram message parsing failed!\n%s" % '\n'.join([event, trace, str(chat_info)]))
    try:
        client.session.save()
    except Exception as e:
        app_log.warning("Could not save session:\n" + e)


@client.on(events.MessageRead())
async def callback_read(event):
    app_log.debug(event)
    chat_info = {
        'ID': event.max_id,
        'Status': 'read'
    }
    if 'original_update' in event:
        try:
            if event.outbox:
                chat_info.update(await get_outgoing_info(client, app_log, event.original_update.peer))
            else:
                chat_info.update(await get_incoming_info(client, app_log, event.original_update.peer))
            app_log.info(chat_info)
            sink_sms.send_dict(IDENTIFIER, None, chat_info)
        except Exception as e:
            event = event.stringify()
            trace = traceback.format_exc()
            app_log.warning(event, exc_info=True)
            app_log.warning(str(chat_info))
            sink_sms.send_notif("Telegram read-message parsing failed!\n%s" % '\n'.join([event, trace, str(chat_info)]))
    else:
        app_log.warning("MessageRead-Event has no 'original_update'!:\n" + event.stringify())


@client.on(events.MessageDeleted())
async def callback_delete(event):
    ids = event.deleted_ids
    app_log.info("Deleted messages with IDs %s!" % str(ids))
    for id in ids:
        sink_sms.send_dict(IDENTIFIER, None, {
            'ID': id,
            'Status': 'deleted'
        })


async def main():
    app_log.info("Catching up..")

    # def handler(signum, frame):
    #     if signum == signal.SIGALRM:
    #       app_log.info("Got a signal for timeout, raising exception")
    #       raise Exception("Function timed out!")
    # signal.signal(signal.SIGALRM, handler)
    # signal.alarm(10)
    try:
        await asyncio.wait_for(client.catch_up(), timeout=30)
    except Exception as e:
        app_log.warning("client.catch_up failed with Exception: %s" % e, exc_info=True)
    # signal.alarm(0)

    app_log.info("Listening to messages..")

    await client.run_until_disconnected()


if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



#!/usr/bin/python
import subprocess, datetime, os, sys, re, json
from smsgateway import sink_sms
from smsgateway.config import *
from smsgateway.sources import utils

app_log = utils.setup_logging("signal")

IDENTIFIER = "SG"

def listen(groups={}):
    app_log.info("Starting signal-cli..")
    proc = subprocess.Popen([utils.SIGNAL_CLI_PATH, '-u', utils.SIGNAL_NUMBER, 'receive', '--json', '-t', str(12 * 3600)], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=sys.stdout.buffer)

    while proc.poll() == None:
        line = proc.stdout.readline().decode('UTF-8')
        app_log.debug(line)
        #proc.stdin.write(bytes('\n', 'UTF-8'))
        if line != '':
            try:
                j = json.loads(line)
                envelope = j['envelope']
                source = envelope['source']
                chat_info = {
                    'ID': envelope['timestamp'],
                }

                dataMsg = envelope['dataMessage']
                syncMsg = envelope['syncMessage']
                info = dataMsg if dataMsg else syncMsg if syncMsg else None
                if info:
                    if 'sentMessage' in info and info['sentMessage']:
                        info = info['sentMessage']
                    if 'message' in info:
                        msg = info['message']
                        if 'groupInfo' in info and info['groupInfo']:
                            groupInfo = info['groupInfo']
                            groupID = groupInfo['groupId']
                            chat_info.update({
                                'type': 'Group',
                                'groupID': groupID,
                            })
                            if groupInfo['type'] == 'UPDATE' and 'name' in groupInfo and groupInfo['name']: # new/changed group
                                if not msg:
                                    if groupID in groups:
                                        msg = 'Group was changed!'
                                    else:
                                        msg = 'Group was created!'
                                    if 'members' in groupInfo:
                                        msg += '\nMembers: ' + ', '.join(groupInfo['members'])
                                groups[groupID] = groupInfo['name']
                            if groupID in groups:
                                chat_info['to'] = groups[groupID]
                            else:
                                app_log.warning(f'Group with ID {groupID} not in group list!')
                            if source != utils.SIGNAL_NUMBER:
                                chat_info.update({
                                    'from': source,
                                    'phone': source,
                                })
                        elif source == utils.SIGNAL_NUMBER and 'destination' in info:  # from me
                            chat_info.update({
                                'to': info['destination'],
                                'phone': info['destination'],
                            })
                        else:
                            chat_info.update({
                                'from': source,
                                'phone': source,
                            })
                        if msg and ('from' in chat_info or 'to' in chat_info):
                            sink_sms.send_dict(IDENTIFIER, msg, chat_info)
                else:
                    app_log.info("Unknown message: %s" % line)
            except KeyError:
                app_log.warning("KeyError in message \"%s\"" % line, exc_info=True)
            # with open ('/var/log/signal.log', 'a') as f:
            #   f.write("%s\n" % line)


def getGroups():
    app_log.info("Getting groups from signal-cli..")
    groups = {}
    try:
        output = subprocess.check_output([utils.SIGNAL_CLI_PATH, '-u', utils.SIGNAL_NUMBER, 'listGroups'], stderr=subprocess.STDOUT, text=True, timeout=300)
        for line in output.splitlines():
            m = re.match('Id: (?P<id>[^ ]+) +Name: (?P<name>[^ ]+) +Active: (?P<active>[^ ]+)$', line)
            if m:
                groupID = m.group('id')
                groupName = m.group('name')
                groups[groupID] = groupName
                app_log.info(f'{groupID}: {groupName}')
            else:
                app_log.warning(f'Couldn\' group definition:\n{line}')
    except subprocess.CalledProcessError  as e:
        app_log.warning(f'Couldn\'t get groups from signal :(\nError:\n{e.output}')
    return groups


groups = getGroups()
listen(groups)

# {
#   "envelope": {
#     "source": "+491743624125",
#     "sourceDevice": 1,
#     "relay": null,
#     "timestamp": 1572988173059,
#     "isReceipt": false,
#     "dataMessage": {
#       "timestamp": 1572988173059,
#       "message": "ugh",
#       "expiresInSeconds": 0,
#       "attachments": [],
#       "groupInfo": null
#     },
#     "syncMessage": null,
#     "callMessage": null,
#     "receiptMessage": null
#   }
# }

# {
#   "envelope": {
#     "source": "+491743624125",
#     "sourceDevice": 1,
#     "relay": null,
#     "timestamp": 1572987164644,
#     "isReceipt": false,
#     "dataMessage": null,
#     "syncMessage": {
#       "sentMessage": {
#         "timestamp": 1572987164644,
#         "message": "test",
#         "expiresInSeconds": 604800,
#         "attachments": [],
#         "groupInfo": null,
#         "destination": "+4915756542328"
#       },
#       "blockedNumbers": null,
#       "readMessages": null,
#       "type": null
#     },
#     "callMessage": null,
#     "receiptMessage": null
#   }
# }

# Group created
# {
#   "envelope": {
#     "source": "+491743624125",
#     "sourceDevice": 1,
#     "relay": null,
#     "timestamp": 1572988370215,
#     "isReceipt": false,
#     "dataMessage": {
#       "timestamp": 1572988370215,
#       "message": "",
#       "expiresInSeconds": 0,
#       "attachments": [],
#       "groupInfo": {
#         "groupId": "eEQ4OfkXuF4/gPQKlaDX8A==",
#         "members": [
#           "+491743624125",
#           "+4915771087176"
#         ],
#         "name": "aha",
#         "type": "UPDATE"
#       }
#     },
#     "syncMessage": null,
#     "callMessage": null,
#     "receiptMessage": null
#   }
# }

# Group message
# {
#   "envelope": {
#     "source": "+491743624125",
#     "sourceDevice": 1,
#     "relay": null,
#     "timestamp": 1572988387668,
#     "isReceipt": false,
#     "dataMessage": {
#       "timestamp": 1572988387668,
#       "message": "lol",
#       "expiresInSeconds": 0,
#       "attachments": [],
#       "groupInfo": {
#         "groupId": "eEQ4OfkXuF4/gPQKlaDX8A==",
#         "members": null,
#         "name": null,
#         "type": "DELIVER"
#       }
#     },
#     "syncMessage": null,
#     "callMessage": null,
#     "receiptMessage": null
#   }
# }



import re, subprocess, json, os
from smsgateway.sources.sms import command_list
from smsgateway.config import *
from smsgateway import sink_sms
from smsgateway.sources.utils import *

import logging
from logging.handlers import RotatingFileHandler

def init():
    global app_log, command_regex

    app_log = setup_logging("signal-send")

    command_regex = re.compile('^(?P<command>[a-zA-Z ]+)$')


def check(cmd, multiline):
    init()
    # app_log.info("Checking %s" % cmd)
    if cmd.lower() == 'sg' and multiline:
      return True
    else:
      return False


def run(lines):
    init()

    app_log.info("Forwarding Signal Message")
    message = ""
    messageStarted = False
    to = None
    for line in lines[1:]: # skip IDENTIFIER
        if messageStarted:
            if message:
                message += "\n"
            message += f"{line}"
        elif not line.strip(): # empty line
            messageStarted = True
        else:
            mTo = re.match("^To: (.*)$", line)
            if mTo:
                to = mTo.group(1).strip()
            else:
                app_log.warning(f"Unkown header: {line}!")
    #toL = lines[1]
    #m = re.match("To:? (.*)$", toL)
    if to:
      #to = m.group(1).replace(' ', '')
      #app_log.info("Matched To: %s" % to)
      #msg = '\n'.join(lines[2:])

      app_log.info("Sending Signal msg:\n%s" % message)
      try:
          args = [SIGNAL_CLI_PATH, '-u', SIGNAL_NUMBER, 'send', '-m', message, to]
          app_log.info("Calling %s" % str(args))
          res = subprocess.call(args, timeout=30000)
      except subprocess.TimeoutExpired as e:
          msg = "Timeout when sending SG to %s: %s" % (to, e)
          app_log.error(msg)
          return msg
      except subprocess.CalledProcessError as e:
          msg = "Failed to send SG to %s: %s" % (to, e.output)
          app_log.error(msg)
          return msg
      if res == 0:
          ret = "Sent SG to %s!" % to
      else:
          ret = "Failed to send SG to %s!" % to
    else:
      ret = "Couldn't match To: %s" % '\n'.join(lines)
      app_log.info(ret)
    return ret

command_list.append({
    'name' : 'SG-Forwarder',
    'check': check,
    'run': run
})



#!/usr/bin/python3
import os, argparse, re, subprocess, time, traceback
import importlib
from smsgateway import sink_sms
import smsgateway.config as conf
from smsgateway.sources.utils import *
from cryptography.fernet import Fernet

command_list = []

def init():
    global app_log, src_path
    app_log = setup_logging("sms")
    src_path = os.path.dirname(os.path.abspath(__file__))

def handleCommand(mods, text):
  if conf.KEY and text.startswith("%8%"):
      text = text.strip()[3:]
      print(f'Trying to decrypt "{text}"')
      f = Fernet(conf.KEY)
      decrypted = f.decrypt(text.encode("utf-8"))
      text = decrypted.decode("utf-8")
  lines = text.strip().split('\n')
  cmd = lines[0].lower().strip()
  ret = "Unknown Command:\n%s" % text
  to = conf.CONTROL_PHONES[0]
  app_log.info("Message has %s lines" % len(lines))
  #if len(lines) == 1:
  app_log.info("Commands: %s" % str(mods))
  multiline = len(lines) > 1
  for c in mods:
      if c.check(cmd, multiline):
          print(f"Mod {c} matched :)")
          try:
              ret = c.run(lines)
          except Exception as e:
              trace = traceback.format_exc()
              msg = "Run failed for command %s:\n%s\n%s" % (cmd, e, trace)
              app_log.error(msg)
              sink_sms.send_notif(msg)
          break
  # elif len(lines) > 2:
  #   if cmd == "SMS":
  #     print("Forwarding SMS")
  #     toL = lines[1]
  #     m = re.match("To: ([0-9 +]+)", toL)
  #     if m:
  #       to = m.group(1).replace(' ', '')
  #       print("Matched To: %s" % to)
  #       ret = '\n'.join(lines[2:])
  #     else:
  #       print("Couldn't match To: %s" % to)
  if ret:
    sink_sms.send_to(to, ret)

def readSMS(filename, received=True):
    direction = "From" if received else "To"
    address = None
    textStarted = False
    decodeUCS2 = False
    text = ''
    decoded_lines = []
    data = {}
    with open(filename, 'rb') as f:
      app_log.info("Opened received SMS")
      lines = f.readlines()
      for line in lines:
          if textStarted == True and decodeUCS2:
            line = line.decode("utf-16-be")
          else:
            line = line.decode("iso8859-15")
          line = line.rstrip()
          decoded_lines.append(line)
          if textStarted == True:
              text += "%s\n" % line
          else:
              m = re.match(f"{direction}: ([0-9]+)", line)
              m2 = re.match(f"{direction}: ([A-Za-z0-9]+)", line)
              mAlpha = re.match("Alphabet: ([A-Z0-9]+)", line)
              if mAlpha:
                  alpha = mAlpha.group(1)
                  if alpha == "UCS2":
                      decodeUCS2 = True
                  elif alpha == "ISO":
                      decodeUCS2 = False
              elif m or m2:
                  if m:
                    address = "+%s" % m.group(1)
                  else:
                    address = "%s" % m2.group(1)
                    app_log.info("Number is not numeric!")
                  app_log.info(f"{direction}: %s" % address)
              elif line == "":
                  textStarted = True
              else:
                  m = re.match(f"([^:]+): (.+)", line)
                  if m:
                      data[m.group(1)] = m.group(2)
    data.update({
      'address': address,
      'text': text,
      'lines': decoded_lines
    })
    return data

def handleSMS(mods, f):
    app_log.info("Received SMS, file: %s" % f)
    data = readSMS(f, received=True)
    # From = None
    # textStarted = False
    # decodeUCS2 = False
    # text = ''
    # with open(f, 'rb') as f:
    #   app_log.info("Opened received SMS")
    #   lines = f.readlines()
    #   for line in lines:
    #     if textStarted == True and decodeUCS2:
    #       line = line.decode("utf-16-be")
    #     else:
    #       line = line.decode("iso8859-15")
    #
    #     line = line.rstrip()
    #     m = re.match("From: ([0-9]+)", line)
    #     m2 = re.match("From: ([A-Za-z0-9]+)", line)
    #     mAlpha = re.match("Alphabet: ([A-Z0-9]+)", line)
    #     if textStarted == True:
    #         text += "%s\n" % line
    #     elif m or m2:
    #         if m:
    #           From = "+%s" % m.group(1)
    #         else:
    #           From = "%s" % m2.group(1)
    #           app_log.info("Number is not numeric!")
    #         app_log.info("From: %s" % From)
    #     elif mAlpha:
    #         alpha = mAlpha.group(1)
    #         if alpha == "UCS2":
    #             decodeUCS2 = True
    #         elif alpha == "ISO":
    #             decodeUCS2 = False
    #     elif line == "":
    #         textStarted = True
    From = data['address']
    text = data['text']
    if From and text:
      if From in ['+%s' % num for num in conf.CONTROL_PHONES]:
        handleCommand(mods, text)
      else: # SMS from someone else
        app_log.info("Sending SMS")
        sink_sms.send("SMS", text, From)
    else:
      app_log.error("Couldn't parse incoming sms!")

def resendSMS(f):
  # textStarted = False
  # To = None
  # textStarted = None
  # wholeText = ''
  # text = ''
  # with open(f, 'r', encoding="8859") as f:
  #   app_log.info("Opened failed SMS")
  #   for line in f:
  #     line = line.rstrip()
  #     wholeText += line
  #     if textStarted == True:
  #       text += "%s\n" % line
  #     else:
  #       m = re.match("To: ([0-9]+)", line)
  #       m_fail = re.match("Fail_reason: (.*)", line)
  #       if m:
  #         To = "+%s" % m.group(1)
  #       elif m_fail:
  #         fail_reason = m_fail.group(1)
  #         app_log.warning("Fail reason: %s" % fail_reason)
  #       elif line == "":
  #         textStarted = True

    data = readSMS(f, received=False)
    To = data['address']
    text = data['text']
    fail_reason = "N/A"
    if 'Fail_reason' in data:
      fail_reason = data['Fail_reason']
    if To and text:
      if To in conf.CONTROL_PHONES:
        app_log.warning("Resending SMS to CONTROL_PHONE")
        new_text = "RESEND: %s\n%s" % (fail_reason, text)
        sink_sms.send_notif(new_text)
      else:
        app_log.warning("Resending SMS to %s" % To)
        sink_sms.send_to(To, text)
        new_text = "RESEND: %s\n%s" % (fail_reason, text)
        sink_sms.send_notif(new_text)
    else:
      new_text = "Invalid failed SMS:\n%s" % '\n'.join(data['lines'])
      app_log.error(new_text)
      sink_sms.send_notif(new_text)

def main(mods):
    app_log.info(src_path)
    parser = argparse.ArgumentParser()
    parser.add_argument("event")
    parser.add_argument("file")
    args = parser.parse_args()

    if not args.event:
      app_log.error("Unknown command, use -h for help")
    else:
      if args.event == "RECEIVED":
        handleSMS(mods, args.file)
      elif args.event == 'FAILED':
        if args.file:
          app_log.warning("SMS failed. Process:")
          app_log.warning("- Stopping smstools.. ")
          (result, out) = run_cmd([conf.SUDO_PATH, conf.SYSTEMCTL_PATH, 'stop', 'smstools'])
          if result == 0:
            app_log.info("Success!")
          else:
            app_log.error("Error:\n%s" % out)

          app_log.info("- Resending: %s" % args.file)
          resendSMS(args.file)
          app_log.warning("- Reboot!")
          for handler in app_log.handlers:
            handler.flush()
          app_log.warning('')
          (res, out) = run_cmd([conf.SUDO_PATH, conf.REBOOT_PATH], "Reboot")
          if res != 0:
            app_log.error("Reboot failed: %s" % out)
        else:
          app_log.error("Got FAILED event, but no file!?")
      else:
        app_log.error("Event is %s, expected RECEIVED" % args.event)

if __name__ == '__main__':
    init()
    try:
      mods = []
      mod_specs = []

      for file in os.listdir(os.path.join(src_path, 'commands')):
            ext_file = os.path.splitext(file)

            if ext_file[1] == '.py' and not ext_file[0] == '__init__':
                mod_name = 'smsgateway.sources.commands.' + ext_file[0]
                spec = importlib.util.find_spec(mod_name)
                if spec not in mod_specs:
                  mod_specs.append(spec)
                  app_log.info("Importing %s" % ext_file[0])
                  m = importlib.import_module(mod_name)
                  mods.append(m)
                  # mod_names.add(mod_name)
                  # print(mod_names)

      main(mods)
    except Exception as e:
      app_log.error("Execution failed: %s" % e, exc_info=True)



import asyncio, json, os, sys, getpass
import time
from datetime import datetime

from nio import AsyncClient, LoginResponse, ClientConfig

from smsgateway import sink_sms
from smsgateway.sources.utils import *
from smsgateway.config import *

from nio import AsyncClient, MatrixRoom, RoomMessageText

CONFIG_DIR = user_config_dir(appname="smsgateway", appauthor="craeckie")
CONFIG_FILE = os.path.join(CONFIG_DIR, "matrix-token.json")
STORE_DIR = os.path.join(CONFIG_DIR, 'matrix-store')

IDENTIFIER = "MX"
User = None

app_log = setup_logging("matrix")

async def message_callback(room: MatrixRoom, event: RoomMessageText) -> None:
    app_log.info(
        f"Message received in room {room.display_name}\n"
        f"{room.user_name(event.sender)} | {event.body}"
    )
    is_group = len(room.users.keys()) > 2 or room.is_named

    if is_group and not room.is_named:
        room_name = ', '.join([user.display_name for id, user in room.users.items() if id != room.own_user_id])
    else:
        room_name = room.display_name

    chat_info = {
        'ID': event.server_timestamp,
        'date': datetime.utcfromtimestamp(event.server_timestamp / 1000),
        'type': 'Group' if is_group else 'User'
    }
    if event.sender == room.own_user_id:  # I sent this message
        if is_group:
            chat_info.update({
                'to_id': room.room_id,
                'to': room_name
            })
        else:
            to_id = list(filter(lambda u: u != User, room.users))[0]
            to_user = room.users[to_id]
            chat_info.update({
                'to_id': to_user.user_id,
                'to': to_user.display_name
            })

    else:
        from_user = [user for u_id, user in room.users.items() if u_id == event.sender][0]
        chat_info.update({
            'from_id': from_user.user_id,
            'from': from_user.display_name
        })
        if is_group:
            chat_info.update({
                'to_id': room.room_id,
                'to': room_name
            })
    #if event.
    #chat_info['edit']
    sink_sms.send_dict(IDENTIFIER, event.body, chat_info)


async def main() -> None:
    global MATRIX_HS_URL, User
    if STORE_DIR and not os.path.isdir(STORE_DIR):
        os.mkdir(STORE_DIR)

    client = None
    client_config = ClientConfig(store_sync_tokens=True)
    # If there are no previously-saved credentials, we'll use the password
    if not os.path.exists(CONFIG_FILE):
        app_log.info("First time use. Did not find credential file. Using supplied "
              "homeserver, user, and password to create credential file.")
        if not (MATRIX_HS_URL.startswith("https://")
                or MATRIX_HS_URL.startswith("http://")):
            MATRIX_HS_URL = "https://" + MATRIX_HS_URL

        User, Pass = MATRIX_CREDENTIALS
        client = AsyncClient(MATRIX_HS_URL, User, config=client_config, store_path=STORE_DIR)

        device_name = "SMSGateway"

        resp = await client.login(Pass, device_name=device_name)

        if (isinstance(resp, LoginResponse)):
            write_details_to_disk(resp, MATRIX_HS_URL, STORE_DIR)
        else:
            print(f"homeserver = \"{MATRIX_HS_URL}\"; user = \"{User}\"")
            print(f"Failed to log in: {resp}")
            sys.exit(1)

    # Otherwise the config file exists, so we'll use the stored credentials
    else:
        # open the file in read-only mode
        with open(CONFIG_FILE, "r") as f:
            config = json.load(f)
            client = AsyncClient(config['homeserver'], config=client_config, store_path=config['store_path'])

            client.access_token = config['access_token']
            client.user_id = config['user_id']
            client.device_id = config['device_id']

    # If you made a new room and haven't joined as that user, you can use
    # await client.join("your-room-id")

    # Load devices & keys
    #client.load_store()
    

    
    # Trust all devices
    #dev_response = await client.devices()
    #print(dev_response.devices)
    # for device in dev_response.devices:
    #     print("Trusting device " + device.display_name)
    #     client.verify_device(device)


    client.add_event_callback(message_callback, RoomMessageText)

    # await client.room_send(
    #     # Watch out! If you join an old room you'll see lots of old messages
    #     room_id="!hBplPMIfOEdWDvufsV:matrix.sanemind.de",
    #     message_type="m.room.message",
    #     content={
    #         "msgtype": "m.text",
    #         "body": "Hello world!"
    #     }
    # )

    await client.sync_forever(full_state=True)

def write_details_to_disk(resp: LoginResponse, homeserver, store_path) -> None:
    """Writes the required login details to disk so we can log in later without
    using a password.

    Arguments:
        resp {LoginResponse} -- the successful client login response.
        homeserver -- URL of homeserver, e.g. "https://matrix.example.org"
    """
    # open the config file in write-mode
    with open(CONFIG_FILE, "w") as f:
        # write the login details to disk
        json.dump(
            {
                "homeserver": homeserver,  # e.g. "https://matrix.example.org"
                "user_id": resp.user_id,  # e.g. "@user:example.org"
                "device_id": resp.device_id,  # device ID, 10 uppercase letters
                "access_token": resp.access_token,  # cryptogr. access token
                "store_path": store_path  # directory to store keys
            },
            f
        )



asyncio.get_event_loop().run_until_complete(main())



import re, subprocess
from smsgateway.sources.sms import command_list
from smsgateway.config import *
from smsgateway.sources.utils import *

service_commands = ['start', 'restart', 'stop', 'enable', 'disable', 'status']
service_regex = re.compile('^(?P<command>[a-zA-Z]+) (?P<service>.+)$')


def run_cmd(args):
    return subprocess.check_output(args, stderr=subprocess.STDOUT).decode('UTF-8').strip()

def match(cmd):
    m = service_regex.match(cmd)
    c = m.groupdict()['command']
    s = m.groupdict()['service']
    return (c, s)

def check(cmd, multiline):
    # print("Checking %s" % cmd)

    if service_regex.match(cmd):
      print("Service RE matches!")
      (c, s) = match(cmd)
      if c in service_commands and s in SERVICES:
        print("Command: %s, Service: %s" % (c, s))
        return True
    return False
def run(lines):
    cmd = lines[0]
    (c, s) = match(cmd)

    if c in service_commands and s in SERVICES:
      try:
        out = run_cmd([SUDO_PATH, SYSTEMCTL_PATH, c, s])

        ret = "%s: OK!\n%s" % (cmd, out)
      except subprocess.CalledProcessError as e:
        ret = "%s failed:\n%s" % (cmd, e.output.decode('UTF-8').strip())
    else:
      ret = "Unknown command or service:\n%s" % cmd
    return ret

command_list.append({
    'name' : 'Services',
    'check': check,
    'run': run
})
